@@ -63,7 +63,7 @@ else:
     from io import StringIO
 
 
-__version__ = '0.3.1'
+__version__ = '0.4.2'
 
 
 verbose = False
@@ -152,43 +152,40 @@ class Graph(object):
         return True
 
 
-class Hash1(object):
+class StrSaltHash(object):
     """
     Random hash function generator.
-    For simplicity and speed, this doesn't implement any byte-level hashing
-    scheme.  Instead, a random string is generated and prefixing to key,
-    and then Python's hashing function is used.
+    Simple byte level hashing: each byte is multiplied to another byte from
+    a random string of characters, summed up, and finally modulo NG is
+    taken.
     """
+    chars = string.ascii_letters + string.digits
+
     def __init__(self, N):
         self.N = N
-        self.salt = random.randint(0, 1 << 31)
-
-    def DEKhash(self, x, s):
-        for c in s:
-            x = ((x << 5) ^ (x >> 27) ^ ord(c)) % (1 << 31)
-        return x
+        self.salt = ''
 
     def __call__(self, key):
-        return self.DEKhash(self.salt, str(key)) % self.N
+        while len(self.salt) < len(key): # add more salt as necessary
+            self.salt += random.choice(self.chars)
+
+        return sum(ord(self.salt[i]) * ord(c)
+                   for i, c in enumerate(key)) % self.N
 
     template = """
-def DEKhash(x, s):
-    for c in s:
-        x = ((x << 5) ^ (x >> 27) ^ ord(c)) % (1 << 31)
-    return x
+def hash_f(key, T):
+    return sum(ord(T[i % $NS]) * ord(c) for i, c in enumerate(key)) % $NG
 
 def perfect_hash(key):
-    return (G[DEKhash($S1, key) % $NG] +
-            G[DEKhash($S2, key) % $NG]) % $NG
+    return (G[hash_f(key, "$S1")] +
+            G[hash_f(key, "$S2")]) % $NG
 """
 
-class Hash2(object):
+class IntSaltHash(object):
     """
     Random hash function generator.
     Simple byte level hashing, each byte is multiplied in sequence to a table
-    containing random numbers modulo N, and then these products are summed up.
-    The table with random numbers is dynamically expanded whenever
-    a key longer than the current table size is encountered.
+    containing random numbers, summed tp, and finally modulo NG is taken.
     """
     def __init__(self, N):
         self.N = N
@@ -213,50 +210,21 @@ def perfect_hash(key):
     return (G[hash_f(key, S1)] + G[hash_f(key, S2)]) % $NG
 """
 
-class Hash3(object):
-    """
-    Random hash function generator.
-    """
-    def __init__(self, N):
-        self.N = N
-        self.salt = random.randint(32, 127)
+def builtin_template(Hash):
+    return """\
+# =======================================================================
+# ================= Python code for perfect hash function ===============
+# =======================================================================
 
-    def __call__(self, key):
-        return sum((self.salt ^ ord(c)) * (i + 1)
-                   for i, c in enumerate(key)) % self.N
+G = [$G]
+""" + Hash.template + """
+# ============================ Sanity check =============================
 
-    template = """
-def hash_f(key, T):
-    return sum((T ^ ord(c)) * (i + 1) for i, c in enumerate(key)) % $NG
+K = [$K]
+assert len(K) == $NK
 
-def perfect_hash(key):
-    return (G[hash_f(key, $S1)] + G[hash_f(key, $S2)]) % $NG
-"""
-
-class Hash4(object):
-    """
-    Random hash function generator.
-    """
-    chars = string.ascii_letters + string.digits
-
-    def __init__(self, N):
-        self.N = N
-        self.salt = ''
-
-    def __call__(self, key):
-        while len(self.salt) < len(key): # add more salt as necessary
-            self.salt += random.choice(self.chars)
-
-        return sum(ord(self.salt[i]) * ord(c)
-                   for i, c in enumerate(key)) % self.N
-
-    template = """
-def hash_f(key, T):
-    return sum(ord(T[i % $NS]) * ord(c) for i, c in enumerate(key)) % $NG
-
-def perfect_hash(key):
-    return (G[hash_f(key, "$S1")] +
-            G[hash_f(key, "$S2")]) % $NG
+for h, k in enumerate(K):
+    assert perfect_hash(k) == h
 """
 
 
@@ -264,7 +232,7 @@ class TooManyInterationsError(Exception):
     pass
 
 
-def generate_hash(keys, Hash=Hash4):
+def generate_hash(keys, Hash=StrSaltHash):
     """
     Return hash functions f1 and f2, and G for a perfect minimal hash.
     Input is an iterable of 'keys', whos indicies are the desired hash values.
@@ -273,36 +241,43 @@ def generate_hash(keys, Hash=Hash4):
     """
     if not isinstance(keys, (list, tuple)):
         raise TypeError("list or tuple expected")
-    if len(keys) != len(set(keys)):
+    NK = len(keys)
+    if NK != len(set(keys)):
         raise ValueError("duplicate keys")
     for key in keys:
         if not isinstance(key, str):
             raise TypeError("key a not string: %r" % key)
+    if NK > 10000 and Hash == StrSaltHash:
+        print("""\
+WARNING: You have %d keys.
+         Using --hft=1 is likely to fail for so many keys.
+         Please use --hft=2 instead.
+""" % NK)
 
-    # N is the number of vertices in the graph G
-    N = len(keys) + 1
+    # the number of vertices in the graph G
+    NG = NK + 1
     if verbose:
-        print('N = %i' % N)
+        print('NG = %d' % NG)
 
     trial = 0  # Number of trial graphs so far
     while True:
-        if (trial % trials) == 0:   # trials failures, increase N slightly
+        if (trial % trials) == 0:   # trials failures, increase NG slightly
             if trial > 0:
-                N = max(N + 1, int(1.05 * N))
+                NG = max(NG + 1, int(1.05 * NG))
             if verbose:
-                sys.stdout.write('\nGenerating graphs N = %i ' % N)
+                sys.stdout.write('\nGenerating graphs NG = %d ' % NG)
         trial += 1
 
-        if N > 100 * (len(keys) + 1):
-            raise TooManyInterationsError("keys=%r" % keys)
+        if NG > 100 * (NK + 1):
+            raise TooManyInterationsError("%d keys" % NK)
 
         if verbose:
             sys.stdout.write('.')
             sys.stdout.flush()
 
-        G = Graph(N)   # Create graph with N vertices
-        f1 = Hash(N)   # Create 2 random hash functions
-        f2 = Hash(N)
+        G = Graph(NG)   # Create graph with NG vertices
+        f1 = Hash(NG)   # Create 2 random hash functions
+        f2 = Hash(NG)
 
         # Connect vertices given by the values of the two hash functions
         # for each key.  Associate the desired hash value with each edge.
@@ -316,15 +291,15 @@ def generate_hash(keys, Hash=Hash4):
             break
 
     if verbose:
-        print('\nAcyclic graph found after %i trials.' % trial)
-        print('N = %i' % N)
+        print('\nAcyclic graph found after %d trials.' % trial)
+        print('NG = %d' % NG)
 
     # Sanity check the result by actually verifying that all the keys
     # hash to the right value.
     for hashval, key in enumerate(keys):
         assert hashval == (
             G.vertex_values[f1(key)] + G.vertex_values[f2(key)]
-        ) % N
+        ) % NG
 
     if verbose:
         print('OK')
@@ -369,10 +344,10 @@ class Format(object):
         return aux.getvalue()
 
 
-def generate_code(keys, template, Hash=Hash4, options=None):
+def generate_code(keys, Hash=StrSaltHash, template=None, options=None):
     """
     Takes a list of key value pairs and inserts the generated parameter
-    lists into the 'template' strinng.  'Hash' is the random hash function
+    lists into the 'template' string.  'Hash' is the random hash function
     generator, and the optional keywords are formating options.
     The return value is the substituted code template.
     """
@@ -385,6 +360,9 @@ def generate_code(keys, template, Hash=Hash4, options=None):
     except TypeError:
         salt_len = None
 
+    if template is None:
+        template = builtin_template(Hash)
+
     if options is None:
         fmt = Format()
     else:
@@ -437,7 +415,7 @@ def read_table(filename, options):
         try:
             key = row[options.keycol - 1]
         except IndexError:
-            sys.exit("%s:%i: Error: Cannot read key, not enough columns." %
+            sys.exit("%s:%d: Error: Cannot read key, not enough columns." %
                      (filename, n + 1))
 
         keys.append(key)
@@ -460,23 +438,6 @@ def read_template(filename):
         sys.exit("Error: Could not open `%s' for reading." % filename)
 
 
-def builtin_template(Hash):
-    return """\
-# =======================================================================
-# ================= Python code for perfect hash function ===============
-# =======================================================================
-
-G = [$G]
-""" + Hash.template + """
-# ============================ Sanity check =============================
-
-K = [$K]
-assert len(K) == $NK
-
-for h, k in enumerate(K):
-    assert perfect_hash(k) == h
-"""
-
 def run_code(code):
     tmpdir = tempfile.mkdtemp()
     path = join(tmpdir, 't.py')
@@ -564,7 +525,7 @@ is processed and the output code is written to stdout.
                       default = 5,
                       type    = "int",
                       help    = "Specifies the number of trials before "
-                                "N is increased.  A small INT will give "
+                                "NG is increased.  A small INT will give "
                                 "compute faster, but the array G will be "
                                 "large.  A large INT will take longer to "
                                 "compute but G will be smaller. "
@@ -573,9 +534,10 @@ is processed and the output code is written to stdout.
 
     parser.add_option("--hft",
                       action  = "store",
-                      default = 4,
+                      default = 1,
                       type    = "int",
-                      help    = "Hash function type INT (see documentation), "
+                      help    = "Hash function type INT.  Possible values "
+                                "are 1 (StrSaltHash) and 2 (IntSaltHash). "
                                 "The default is %default",
                       metavar = "INT")
 
@@ -616,13 +578,9 @@ is processed and the output code is written to stdout.
         parser.error("template filename does not contain 'tmpl'")
 
     if options.hft == 1:
-        Hash = Hash1
+        Hash = StrSaltHash
     elif options.hft == 2:
-        Hash = Hash2
-    elif options.hft == 3:
-        Hash = Hash3
-    elif options.hft == 4:
-        Hash = Hash4
+        Hash = IntSaltHash
     else:
         parser.error("Hash function %s not implemented." % options.hft)
 
@@ -643,8 +601,7 @@ is processed and the output code is written to stdout.
     if verbose:
         print("tmpl_file = %r" % tmpl_file)
 
-    template = (read_template(tmpl_file) if tmpl_file else
-                builtin_template(Hash))
+    template = read_template(tmpl_file) if tmpl_file else None
 
     if options.output:
         outname = options.output
@@ -669,7 +626,7 @@ is processed and the output code is written to stdout.
         except IOError:
             sys.exit("Error: Could not open `%s' for writing." % outname)
 
-    code = generate_code(keys, template, Hash, options)
+    code = generate_code(keys, Hash, template, options)
 
     if options.execute or template == builtin_template(Hash):
         if verbose:
